/****************************************************************
Copyright (c) 1998 Sandro Sigala <ssigala@globalnet.it>.
All rights reserved.

Permission to use, copy, modify, and distribute this software
and its documentation for any purpose and without fee is hereby
granted, provided that the above copyright notice appear in all
copies and that both that the copyright notice and this
permission notice and warranty disclaimer appear in supporting
documentation, and that the name of the author not be used in
advertising or publicity pertaining to distribution of the
software without specific, written prior permission.

The author disclaim all warranties with regard to this
software, including all implied warranties of merchantability
and fitness.  In no event shall the author be liable for any
special, indirect or consequential damages or any damages
whatsoever resulting from loss of use, data or profits, whether
in an action of contract, negligence or other tortious action,
arising out of or in connection with the use or performance of
this software.
****************************************************************/

#include "config.h"

#include <kapp.h>
#include <kmenubar.h>
#include <kmsgbox.h>
#include <qaccel.h>
#include <qlabel.h>
#include <qpainter.h>
#include <qpopmenu.h>
#include <qpushbt.h>

#include "kzacman.h"
#include "gamewindow.h"
#include "gamewidget.h"
#include "scoredialog.h"

const int minimum_width = 5 * sprite_width;
const int minimum_height = 3 * sprite_height;
const int default_width = 9 * sprite_width;
const int default_height = 9 * sprite_height;

GameWindow::GameWindow(const char *name)
        : KTopLevelWidget(name)
{
	setCaption("Zacman");

	file_popup = new QPopupMenu();
	file_popup->insertItem("&New Game", this, SLOT(menu_newGame()), CTRL+Key_N);
        pauseID = file_popup->insertItem("&Pause", this, SLOT(menu_pause()), Key_F2);
        file_popup->insertItem("&End Game", this, SLOT(menu_endGame()), CTRL+Key_E);
	file_popup->insertSeparator();
	file_popup->insertItem("&High Scores...", this, SLOT(menu_highScores()));
	file_popup->insertSeparator();
	file_popup->insertItem("&Quit", qApp, SLOT(quit()), CTRL+Key_Q);

	skill_popup = new QPopupMenu();
	beginnerID = skill_popup->insertItem("&Beginner", this, SLOT(menu_beginner()));
	intermediateID = skill_popup->insertItem("&Intermediate", this, SLOT(menu_intermediate()));
	advancedID = skill_popup->insertItem("&Advanced", this, SLOT(menu_advanced()));
	expertID = skill_popup->insertItem("&Expert", this, SLOT(menu_expert()));

	options_popup = new QPopupMenu();
	options_popup->insertItem("Skill &level", skill_popup);
	options_popup->insertSeparator();
	soundsID = options_popup->insertItem("&Sounds", this, SLOT(menu_sounds()));

	help_popup = new QPopupMenu();
	help_popup->insertItem("&Contents", this, SLOT(menu_help()), Key_F1);
	help_popup->insertSeparator();
	help_popup->insertItem("&About KZacman...", this, SLOT(menu_about()));
//	help_popup->insertItem("About &KDE...", this, SLOT(menu_aboutKDE()));

	menu = new KMenuBar(this);
	menu->insertItem("&File", file_popup);
	menu->insertItem("&Options", options_popup);
	menu->insertItem("&Help", help_popup);
	setMenu(menu);
	connect(menu, SIGNAL(moved(menuPosition)),
		this, SLOT(movedMenu(menuPosition)));

	status = new KStatusBar(this);
	status->insertItem("Points left: 999", 1);
	status->insertItem("Elapsed time: 999", 2);
	status->insertItem("Potion: ", 3);
	setStatusBar(status);
	status->changeItem("", 1);
	status->changeItem("", 2);
	status->changeItem("", 3);
 
	game = new GameWidget(this);
	setView(game);
	connect(game, SIGNAL(changedStats(int, int, int)),
		this, SLOT(updateStats(int, int, int)));
	connect(game, SIGNAL(noStats()), this, SLOT(noStats()));

	setMinimumSize(minimum_width, minimum_height + menu->height() + status->height());
	setSizeIncrement(sprite_width, sprite_height);
	resize(default_width, default_height + menu->height() + status->height());

	// Read configuration
	KConfig *config = kapp->getConfig();
	config->setGroup("Options");
	skill = (Skill)config->readNumEntry("Skill", Skill_Beginner);
	switch (skill) {
	case Skill_Beginner:
		skill_popup->setItemChecked(beginnerID, true);
		break;
	case Skill_Intermediate:
		skill_popup->setItemChecked(intermediateID, true);
		break;
	case Skill_Advanced:
		skill_popup->setItemChecked(advancedID, true);
		break;
	case Skill_Expert:
		skill_popup->setItemChecked(expertID, true);
	}

	game->do_sounds = config->readBoolEntry("Sounds", true);
	options_popup->setItemChecked(soundsID, game->do_sounds);
}

void GameWindow::menu_newGame()
{
	file_popup->setItemChecked(pauseID, false);
	game->newGame(skill);
}

void GameWindow::menu_pause()
{
	if (game->in_game) {
		game->in_pause = !game->in_pause;
		file_popup->setItemChecked(pauseID, game->in_pause);
		game->repaint(false);
	}
}

void GameWindow::menu_endGame()
{
	game->in_game = false;
	game->repaint(false);
	file_popup->setItemChecked(pauseID, false);
}

void GameWindow::menu_highScores()
{
	ScoreDialog d(this);
	d.exec();
}

void GameWindow::menu_beginner()
{
	if (skill != Skill_Beginner) {
		skill = Skill_Beginner;
		skill_popup->setItemChecked(beginnerID, true);
		skill_popup->setItemChecked(intermediateID, false);
		skill_popup->setItemChecked(advancedID, false);
		skill_popup->setItemChecked(expertID, false);
		KConfig *config = kapp->getConfig();
		config->setGroup("Options");
		config->writeEntry("Skill", Skill_Beginner);
		if (game->in_game) {
			file_popup->setItemChecked(pauseID, false);
			game->newGame(skill);
		}
	}
}

void GameWindow::menu_intermediate()
{
	if (skill != Skill_Intermediate) {
		skill = Skill_Intermediate;
		skill_popup->setItemChecked(beginnerID, false);
		skill_popup->setItemChecked(intermediateID, true);
		skill_popup->setItemChecked(advancedID, false);
		skill_popup->setItemChecked(expertID, false);
		KConfig *config = kapp->getConfig();
		config->setGroup("Options");
		config->writeEntry("Skill", Skill_Intermediate);
		if (game->in_game) {
			file_popup->setItemChecked(pauseID, false);
			game->newGame(skill);
		}
	}
}

void GameWindow::menu_advanced()
{
	if (skill != Skill_Advanced) {
		skill = Skill_Advanced;
		skill_popup->setItemChecked(beginnerID, false);
		skill_popup->setItemChecked(intermediateID, false);
		skill_popup->setItemChecked(advancedID, true);
		skill_popup->setItemChecked(expertID, false);
		KConfig *config = kapp->getConfig();
		config->setGroup("Options");
		config->writeEntry("Skill", Skill_Advanced);
		if (game->in_game) {
			file_popup->setItemChecked(pauseID, false);
			game->newGame(skill);
		}
	}
}

void GameWindow::menu_expert()
{
	if (skill != Skill_Expert) {
		skill = Skill_Expert;
		skill_popup->setItemChecked(beginnerID, false);
		skill_popup->setItemChecked(intermediateID, false);
		skill_popup->setItemChecked(advancedID, false);
		skill_popup->setItemChecked(expertID, true);
		KConfig *config = kapp->getConfig();
		config->setGroup("Options");
		config->writeEntry("Skill", Skill_Expert);
		if (game->in_game) {
			file_popup->setItemChecked(pauseID, false);
			game->newGame(skill);
		}
	}
}

void GameWindow::menu_sounds()
{
	game->do_sounds = !game->do_sounds;
	options_popup->setItemChecked(soundsID, game->do_sounds);
	KConfig *config = kapp->getConfig();
	config->setGroup("Options");
	config->writeEntry("Sounds", game->do_sounds);
}

void GameWindow::menu_help()
{
	kapp->invokeHTMLHelp("", "");
}

void GameWindow::menu_about()
{
	KMsgBox::message(this, "About KZacman",
			 "KZacman " VERSION "\n\n"
			 "by Sandro Sigala <ssigala@globalnet.it>\n\n"
			 "Feedbacks are welcome!\n",
			 KMsgBox::INFORMATION);
}

void GameWindow::menu_aboutKDE()
{
}

void GameWindow::updateStats(int points, int secs, int potion)
{
	QString p, s, t;
	p.setNum(points);
	status->changeItem(QString("Points left: ") + p, 1);
	s.setNum(secs);
	status->changeItem(QString("Elapsed time: ") + s, 2);
	if (potion > 0) {
		t.setNum(potion);
		status->changeItem(QString("Potion: ") + t, 3);
	} else
		status->changeItem("", 3);
}

void GameWindow::noStats()
{
	status->changeItem("", 1);
	status->changeItem("", 2);
	status->changeItem("", 3);
}

void GameWindow::movedMenu(menuPosition pos)
{
	if (pos == Top) {
		setMinimumSize(minimum_width, minimum_height + menu->height() + status->height());
		resize(width(), height() + menu->height());
	} else {
		setMinimumSize(minimum_width, minimum_height + status->height());
		resize(width(), height() - menu->height());
	}
}
